@@ -25,7 +25,6 @@
 #include <QNetworkCookie>
 #include <QNetworkCookieJar>
 #include <QNetworkProxy>
-#include <QNetworkReply>
 #include <QSslCipher>
 
 #include <KLocalizedString>
@@ -144,7 +143,7 @@ void HTTPProtocol::handleSslErrors(QNetworkReply *reply, const QList<QSslError>
     QList<QSslCertificate> certs = reply->sslConfiguration().peerCertificateChain();
 
     QStringList peerCertChain;
-    for (const QSslCertificate &cert : certs) {
+    for (const QSslCertificate &cert : std::as_const(certs)) {
         peerCertChain += QString::fromUtf8(cert.toPem());
     }
 
@@ -176,7 +175,7 @@ void HTTPProtocol::handleSslErrors(QNetworkReply *reply, const QList<QSslError>
 
     QStringList certificateErrors;
     // encode the two-dimensional numeric error list using '\n' and '\t' as outer and inner separators
-    for (const QSslCertificate &cert : certs) {
+    for (const QSslCertificate &cert : std::as_const(certs)) {
         QString errorStr;
         for (const QSslError &error : std::as_const(sslErrors)) {
             if (error.certificate() == cert) {
@@ -258,8 +257,17 @@ HTTPProtocol::Response HTTPProtocol::makeDavRequest(const QUrl &url,
 HTTPProtocol::Response
 HTTPProtocol::makeRequest(const QUrl &url, KIO::HTTP_METHOD method, QByteArray &inputData, DataMode dataMode, const QMap<QByteArray, QByteArray> &extraHeaders)
 {
+    /* HTTPProtocol::get(...) creates an empty inputData whether or not the calling function
+     * sent data to the request. QNetworkRequest sends "Content-Length: 0" for all requests
+     * when the device != nullptr, even if data is empty. Per RFC9110, "A user agent SHOULD NOT
+     * send a Content-Length header field when the request message does not contain content and
+     * the method semantics do not anticipate such data." Semantically, HTTP_GET, HTTP_HEAD,
+     * and others shouldn't send the header when the data is empty. Workaround that behavior
+     * here until and if Qt is modified. https://bugreports.qt.io/browse/QTBUG-138848 */
+    const bool noBodyWhenEmpty = (method == KIO::HTTP_GET || method == KIO::HTTP_HEAD || method == KIO::HTTP_DELETE);
     QBuffer buffer(&inputData);
-    return makeRequest(url, method, &buffer, dataMode, extraHeaders);
+    QIODevice *bodyDevice = (noBodyWhenEmpty && inputData.isEmpty()) ? nullptr : &buffer;
+    return makeRequest(url, method, bodyDevice, dataMode, extraHeaders);
 }
 
 static QString protocolForProxyType(QNetworkProxy::ProxyType type)
@@ -558,6 +566,7 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
         }
         QUrl newUrl = url;
         newUrl.setPath(newUrl.path() + QLatin1Char('/'));
+        reply->deleteLater();
         return makeRequest(newUrl, method, inputData, dataMode, extraHeaders);
     }
     if (inputData) {
@@ -572,6 +581,29 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
         data(QByteArray());
     });
 
+    const int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    if (reply->error() != QNetworkReply::NoError && statusCode == 0) {
+        reply->deleteLater();
+
+        // anything not listed here gets KIO::ERR_CANNOT_CONNECT
+        static constexpr const std::pair<QNetworkReply::NetworkError, int> qnam2kio_errors[] = {
+            {QNetworkReply::HostNotFoundError, KIO::ERR_UNKNOWN_HOST},
+            {QNetworkReply::TimeoutError, KIO::ERR_SERVER_TIMEOUT},
+            {QNetworkReply::OperationCanceledError, KIO::ERR_USER_CANCELED},
+            {QNetworkReply::TemporaryNetworkFailureError, KIO::ERR_CONNECTION_BROKEN},
+            {QNetworkReply::NetworkSessionFailedError, KIO::ERR_CONNECTION_BROKEN},
+            {QNetworkReply::ProxyConnectionClosedError, KIO::ERR_CONNECTION_BROKEN},
+            {QNetworkReply::ProxyNotFoundError, KIO::ERR_UNKNOWN_PROXY_HOST},
+            {QNetworkReply::ProxyTimeoutError, KIO::ERR_SERVER_TIMEOUT},
+            {QNetworkReply::ProtocolUnknownError, KIO::ERR_UNSUPPORTED_PROTOCOL},
+        };
+
+        const auto it = std::ranges::find_if(qnam2kio_errors, [reply](const auto &m) {
+            return m.first == reply->error();
+        });
+
+        return {0, QByteArray(), it != std::end(qnam2kio_errors) ? (*it).second : KIO::ERR_CANNOT_CONNECT};
+    }
     if (reply->error() == QNetworkReply::AuthenticationRequiredError) {
         reply->deleteLater();
         return {0, QByteArray(), KIO::ERR_ACCESS_DENIED};
@@ -594,8 +626,6 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
         returnData = reply->readAll();
     }
 
-    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-
     setMetaData(QStringLiteral("responsecode"), QString::number(statusCode));
     setMetaData(QStringLiteral("content-type"), readMimeType(reply));
 
@@ -1624,13 +1654,12 @@ static int httpPutError(const QUrl &url, int responseCode, QString *errorString)
 
 KIO::WorkerResult HTTPProtocol::sendHttpError(const QUrl &url, KIO::HTTP_METHOD method, const HTTPProtocol::Response &response)
 {
-    QString errorString;
-    int errorCode = 0;
-
-    if (response.kioCode == KIO::ERR_ACCESS_DENIED) {
+    if (response.kioCode != KJob::NoError) {
         return KIO::WorkerResult::fail(response.kioCode, url.toDisplayString());
     }
 
+    QString errorString;
+    int errorCode = KJob::NoError;
     int responseCode = response.httpCode;
 
     if (method == KIO::HTTP_PUT) {
